@@ -197,10 +197,10 @@ colGrid <- function(...)
 setAutoGridMaker <- function(GRIDMAKER="defaultAutoGrid")
 {
     match.fun(GRIDMAKER)  # sanity check
-    set_user_option("auto.grid.maker", GRIDMAKER)
+    S4Arrays:::set_user_option("auto.grid.maker", GRIDMAKER)
 }
 
-getAutoGridMaker <- function() get_user_option("auto.grid.maker")
+getAutoGridMaker <- function() S4Arrays:::get_user_option("auto.grid.maker")
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -227,7 +227,7 @@ normarg_grid <- function(grid, x)
         if (!is(grid, "ArrayGrid"))
             stop(wmsg("'grid' must be NULL or an ArrayGrid object"))
         if (!identical(refdim(grid), dim(x)))
-            stop(wmsg("'grid' is incompatible with 'x'"))
+            stop(wmsg("supplied 'grid' is incompatible with 'x'"))
     }
     grid
 }
@@ -270,7 +270,7 @@ normarg_sink_grid <- function(grid, sink)
         if (!is(grid, "ArrayGrid"))
             stop(wmsg("'grid' must be NULL or an ArrayGrid object"))
         if (!identical(refdim(grid), dim(sink)))
-            stop(wmsg("'grid' is incompatible with 'sink'"))
+            stop(wmsg("supplied 'grid' is incompatible with 'sink'"))
     }
     grid
 }
