@@ -125,9 +125,9 @@ setMethod("is_sparse", "DelayedNaryIsoOp",
 )
 
 ### 'is_sparse(x)' is assumed to be TRUE and 'index' is assumed to
-### not contain duplicates. See "extract_sparse_array() Terms of Use"
+### not contain duplicates. See "OLD_extract_sparse_array() Terms of Use"
 ### in SparseArraySeed-class.R
-setMethod("extract_sparse_array", "DelayedNaryIsoOp",
+setMethod("OLD_extract_sparse_array", "DelayedNaryIsoOp",
     function(x, index)
     {
         stop("NOT IMPLEMENTED YET!")
