@@ -79,32 +79,38 @@ setAutoBPPARAM <- function(BPPARAM=NULL)
             stop(wmsg("'BPPARAM' must be a BiocParallelParam ",
                       "object from the BiocParallel package"))
     }
-    set_user_option("auto.BPPARAM", BPPARAM)
+    S4Arrays:::set_user_option("auto.BPPARAM", BPPARAM)
 }
 
-getAutoBPPARAM <- function() get_user_option("auto.BPPARAM")
+getAutoBPPARAM <- function() S4Arrays:::get_user_option("auto.BPPARAM")
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Set/get grid context for the current block of a blockApply(),
-### gridApply(), or blockReduce() loop
+### Set/get grid context for the current block of a gridApply(), blockApply(),
+### gridReduce(), or blockReduce() loop
 ###
 
-set_grid_context <- function(effective_grid, current_block_id,
+set_grid_context <- function(effective_grid,
+                             current_block_id,
+                             current_viewport=NULL,
                              envir=parent.frame(1))
 {
-    assign(".effective_grid", effective_grid, envir=envir)
-    assign(".current_block_id", current_block_id, envir=envir)
+    if (!is.null(effective_grid))
+        assign(".effective_grid", effective_grid, envir=envir)
+    if (!is.null(current_block_id))
+        assign(".current_block_id", current_block_id, envir=envir)
+    if (!is.null(current_viewport))
+        assign(".current_viewport", current_viewport, envir=envir)
 }
 
 .backward_compat <- function(envir, funname)
 {
     if (!(is.array(envir) || is(envir, "SparseArraySeed")))
         return(envir)
-    msg <- c("starting with DelayedArray 0.15.12, passing 'block' ",
-             "to ", funname, "() is no longer needed and will soon ",
-             "be considered an error")
-    .Deprecated(msg=c("  ", wmsg(msg)))
+    msg <- c("starting with DelayedArray 0.21.1, passing 'block' ",
+             "to ", funname, "() is no longer needed and is ",
+             "considered an error")
+    .Defunct(msg=c("  ", wmsg(msg)))
     parent.frame(3)
 }
 
@@ -157,6 +163,10 @@ currentBlockId <- function(envir=parent.frame(2))
 currentViewport <- function(envir=parent.frame(2))
 {
     envir <- .backward_compat(envir, "currentViewport")
+    current_viewport <- try(get(".current_viewport", envir=envir,
+                                inherits=FALSE), silent=TRUE)
+    if (!inherits(current_viewport, "try-error"))
+        return(current_viewport)
     effective_grid <- try(effectiveGrid(envir), silent=TRUE)
     if (inherits(effective_grid, "try-error"))
         stop(wmsg(.grid_context_not_found),
@@ -194,19 +204,19 @@ gridApply <- function(grid, FUN, ..., BPPARAM=getAutoBPPARAM(), verbose=NA)
                     appendLF=FALSE)
         }
         viewport <- grid[[bid]]
-        set_grid_context(grid, bid)
+        set_grid_context(grid, bid, viewport)
         ans <- FUN(viewport, ...)
         if (verbose)
             message("OK")
         ans
     }
-    bplapply2(seq_along(grid), FUN_WRAPPER, grid, verbose,
-                               FUN, ..., BPPARAM=BPPARAM)
+    S4Arrays:::bplapply2(seq_along(grid), FUN_WRAPPER, grid, verbose,
+                                          FUN, ..., BPPARAM=BPPARAM)
 }
 
 viewportApply <- function(...)
 {
-    .Deprecated("gridApply")
+    .Defunct("gridApply")
     gridApply(...)
 }
 
@@ -232,7 +242,7 @@ blockApply <- function(x, FUN, ..., grid=NULL, as.sparse=FALSE,
         } else {
             block <- read_block(x, viewport, as.sparse=as.sparse)
         }
-        set_grid_context(effective_grid, current_block_id)
+        set_grid_context(effective_grid, current_block_id, viewport)
         if (verbose)
             message("\\ Processing it ... ", appendLF=FALSE)
         ans <- FUN(block, ...)
@@ -264,7 +274,7 @@ gridReduce <- function(FUN, grid, init, ..., BREAKIF=NULL, verbose=NA)
     nblock <- length(grid)
     for (bid in seq_len(nblock)) {
         viewport <- grid[[bid]]
-        set_grid_context(grid, bid)
+        set_grid_context(grid, bid, viewport)
         if (verbose)
             message("\\ Processing viewport ", bid, "/", nblock, " ... ",
                     appendLF=FALSE)
@@ -282,7 +292,7 @@ gridReduce <- function(FUN, grid, init, ..., BREAKIF=NULL, verbose=NA)
 
 viewportReduce <- function(...)
 {
-    .Deprecated("gridReduce")
+    .Defunct("gridReduce")
     gridReduce(...)
 }
 
@@ -310,7 +320,7 @@ blockReduce <- function(FUN, x, init, ..., BREAKIF=NULL,
         } else {
             block <- read_block(x, viewport, as.sparse=as.sparse)
         }
-        set_grid_context(effective_grid, current_block_id)
+        set_grid_context(effective_grid, current_block_id, viewport)
         if (verbose)
             message("\\ Processing it ... ", appendLF=FALSE)
         init <- FUN(block, init, ...)
@@ -328,7 +338,6 @@ blockReduce <- function(FUN, x, init, ..., BREAKIF=NULL,
         }
     } else {
         BREAKIF_WRAPPER <- BREAKIF
-        
     }
     gridReduce(FUN_WRAPPER, grid, init,
                FUN, x, as.sparse, verbose, verbose_read_block, ...,
@@ -336,6 +345,178 @@ blockReduce <- function(FUN, x, init, ..., BREAKIF=NULL,
 }
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Specialized internal helpers for the 2D case
+###
+
+### We give our preference to "full width" blocks because they avoid walking
+### twice on each "horizontal strip" e.g. in the case of BLOCK_rowVars()
+### when 'center' is not supplied.
+### However we only go for "full width" blocks when we are **absolutely
+### certain** that this is going to play well with the chunk geometry.
+### If "full width" blocks are not possible or too risky, our second choice
+### is to go for the most "horizontally elongated" blocks, because, it's
+### better to have a high number of narrow strips than a small number of
+### thick strips from a parallelization point of view (we can efficiently
+### use more workers).
+### In case of any uncertainty about the physical layout of the data (i.e.
+### if 'chunkdim(x)' is NULL), we just use square blocks to lower the risk
+### of using completely inappropriate block geometry.
+best_grid_for_hstrip_apply <- function(x, grid=NULL)
+{
+    if (!is.null(grid))
+        return(normarg_grid(grid, x))  # only to check the supplied 'grid'
+    if (length(x) == 0L)
+        return(defaultAutoGrid(x))  # one empty block
+    x_chunkdim <- chunkdim(x)
+    if (is.null(x_chunkdim)) {
+        if (is.matrix(x))
+            return(rowAutoGrid(x))  # "full width" blocks
+        ## We're in doubt about how "full width" blocks are going to play
+        ## with the physical layout of the data (e.g. 'x' is a TENxMatrix
+        ## object that was subsetted by row).
+        return(defaultAutoGrid(x, block.shape="hypercube"))  # square blocks
+    }
+    chunk_nrow <- x_chunkdim[[1L]]
+    n <- getAutoBlockSize() /
+         (as.double(get_type_size(type(x))) * chunk_nrow * ncol(x))
+    if (n < 1)
+        return(defaultAutoGrid(x, block.shape="last-dim-grows-first"))
+    block_nrow <- min(as.integer(n) * chunk_nrow, nrow(x))
+    rowAutoGrid(x, nrow=block_nrow)  # "full width" blocks
+}
+
+best_grid_for_vstrip_apply <- function(x, grid=NULL)
+{
+    if (!is.null(grid))
+        return(normarg_grid(grid, x))  # only to check the supplied 'grid'
+    if (length(x) == 0L)
+        return(defaultAutoGrid(x))  # one empty block
+    x_chunkdim <- chunkdim(x)
+    if (is.null(x_chunkdim)) {
+        if (is.matrix(x))
+            return(colAutoGrid(x))  # "full height" blocks
+        ## We're in doubt about how "full height" blocks are going to play
+        ## with the physical layout of the data (e.g. 'x' is a TENxMatrix
+        ## object that was subsetted by row and then transposed).
+        return(defaultAutoGrid(x, block.shape="hypercube"))  # square blocks
+    }
+    chunk_ncol <- x_chunkdim[[2L]]
+    n <- getAutoBlockSize() /
+         (as.double(get_type_size(type(x))) * nrow(x) * chunk_ncol)
+    if (n < 1)
+        return(defaultAutoGrid(x, block.shape="first-dim-grows-first"))
+    block_ncol <- min(as.integer(n) * chunk_ncol, ncol(x))
+    colAutoGrid(x, ncol=block_ncol)  # "full height" blocks
+}
+
+reduce_grid_hstrip <- function(i, grid, x, INIT, INIT_MoreArgs,
+                                           FUN, FUN_MoreArgs,
+                                           FINAL, FINAL_MoreArgs,
+                                           as.sparse, verbose)
+{
+    INIT <- match.fun(INIT)
+    FUN <- match.fun(FUN)
+    if (!is.null(FINAL))
+        FINAL <- match.fun(FINAL)
+    verbose <- normarg_verbose(verbose)
+    init <- do.call(INIT, c(list(i, grid), INIT_MoreArgs))
+    grid_nrow <- nrow(grid)
+    grid_ncol <- ncol(grid)
+    ## Walk on the blocks of the i-th horizontal strip. Sequential.
+    for (j in seq_len(grid_ncol)) {
+        viewport <- grid[[i, j]]
+        block <- read_block(x, viewport, as.sparse=as.sparse)
+        set_grid_context(grid, NULL, viewport)
+        if (verbose)
+            message("Processing block [[", i, "/", grid_nrow, ", ",
+                                           j, "/", grid_ncol, "]] ... ",
+                    appendLF=FALSE)
+        init <- do.call(FUN, c(list(init, block), FUN_MoreArgs))
+        if (verbose)
+            message("OK")
+    }
+    if (!is.null(FINAL))
+        init <- do.call(FINAL, c(list(init), FINAL_MoreArgs))
+    init
+}
+
+reduce_grid_vstrip <- function(j, grid, x, INIT, INIT_MoreArgs,
+                                           FUN, FUN_MoreArgs,
+                                           FINAL, FINAL_MoreArgs,
+                                           as.sparse, verbose)
+{
+    INIT <- match.fun(INIT)
+    FUN <- match.fun(FUN)
+    if (!is.null(FINAL))
+        FINAL <- match.fun(FINAL)
+    verbose <- normarg_verbose(verbose)
+    init <- do.call(INIT, c(list(j, grid), INIT_MoreArgs))
+    grid_nrow <- nrow(grid)
+    grid_ncol <- ncol(grid)
+    ## Walk on the blocks of the j-th vertical strip. Sequential.
+    for (i in seq_len(grid_nrow)) {
+        viewport <- grid[[i, j]]
+        block <- read_block(x, viewport, as.sparse=as.sparse)
+        set_grid_context(grid, NULL, viewport)
+        if (verbose)
+            message("Processing block [[", i, "/", grid_nrow, ", ",
+                                           j, "/", grid_ncol, "]] ... ",
+                    appendLF=FALSE)
+        init <- do.call(FUN, c(list(init, block), FUN_MoreArgs))
+        if (verbose)
+            message("OK")
+    }
+    if (!is.null(FINAL))
+        init <- do.call(FINAL, c(list(init), FINAL_MoreArgs))
+    init
+}
+
+### Walk on the horizontal grid strips.
+### Return a list with one list element per strip.
+hstrip_apply <- function(x, INIT, INIT_MoreArgs, FUN, FUN_MoreArgs,
+                            FINAL=NULL, FINAL_MoreArgs=list(),
+                            grid=NULL, as.sparse=FALSE,
+                            BPPARAM=getAutoBPPARAM(), verbose=NA)
+{
+    grid <- best_grid_for_hstrip_apply(x, grid)
+    if (!(is.logical(as.sparse) && length(as.sparse) == 1L))
+        stop(wmsg("'as.sparse' must be a FALSE, TRUE, or NA"))
+
+    ## Outer loop on the horizontal grid strips. Parallelized.
+    S4Arrays:::bplapply2(seq_len(nrow(grid)),
+        reduce_grid_hstrip, grid, x,
+        INIT, INIT_MoreArgs,
+        FUN, FUN_MoreArgs,
+        FINAL, FINAL_MoreArgs,
+        as.sparse, verbose,
+        BPPARAM=BPPARAM
+    )
+}
+
+### Walk on the vertical grid strips.
+### Return a list with one list element per strip.
+vstrip_apply <- function(x, INIT, INIT_MoreArgs, FUN, FUN_MoreArgs,
+                            FINAL=NULL, FINAL_MoreArgs=list(),
+                            grid=NULL, as.sparse=FALSE,
+                            BPPARAM=getAutoBPPARAM(), verbose=NA)
+{
+    grid <- best_grid_for_vstrip_apply(x, grid)
+    if (!(is.logical(as.sparse) && length(as.sparse) == 1L))
+        stop(wmsg("'as.sparse' must be a FALSE, TRUE, or NA"))
+
+    ## Outer loop on the vertical grid strips. Parallelized.
+    S4Arrays:::bplapply2(seq_len(ncol(grid)),
+        reduce_grid_vstrip, grid, x,
+        INIT, INIT_MoreArgs,
+        FUN, FUN_MoreArgs,
+        FINAL, FINAL_MoreArgs,
+        as.sparse, verbose,
+        BPPARAM=BPPARAM
+    )
+}
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### OLD - Walking on the blocks
 ### OLD -
