@@ -66,7 +66,7 @@ setMethod("realize", "ANY",
         ## the dimnames. See R/writeHDF5Array.R in the HDF5Array package for
         ## more information about this.
         ## TODO: Remove line below when this is addressed.
-        set_dimnames(ans, dimnames(x))
+        S4Arrays:::set_dimnames(ans, dimnames(x))
     }
 )
 
