@@ -5,8 +5,8 @@ test_RleArray <- function()
     A1 <- RleArray(data, c(62500, 400))
     A2 <- RleArray(data, c(62500, 400), chunksize=1e8)
 
-    on.exit(setAutoBlockSize())
-    setAutoBlockSize(10e6)
+    on.exit(suppressMessages(setAutoBlockSize()))
+    suppressMessages(setAutoBlockSize(10e6))
     rs1 <- rowSums(A1)
     rs2 <- rowSums(A2)
     checkIdentical(rs1, rs2)
